/**
***     Copyright (C) 2005 - 2005 Sophia Cradle, Inc.
***     All rights reserved.
***
***     This source code was written by HORIGUCHI Junshi,
***                                 GOTO Hiromichi.
***
***     Project         : CameraSample
***     Codename        : N/A
***     Compiler        : Visual C++ 6.0, ADS 1.1, RVCTB 1.2
***     Package         : N/A
***     File            : Camera.cpp
**/

#include "Camera.hpp"

Camera::Camera(Void)
{
    return;
}

Camera::~Camera(Void)
{
    Terminate();
    return;
}

SFCError Camera::Initialize(SFXRectangle const& rect, NotifySPP spp, VoidPtr reference)
{
    SFCError error = SFERR_NO_ERROR;

    _getable = false;
    // vr[摜ۑp̃rbg}bv𐶐B
    if ((_bitmap = SFXGraphics::GetInstance()->CreateBitmap(rect.GetSize())) != NULL) {
        // W[ƓsB
        error = Resume();
        if (error == SFERR_NO_ERROR) {
            _rect = rect;
            _spp = spp;
            _reference = reference;
        }
    }
    else {
        error = ENOMEMORY;
    }
    if (error != SFERR_NO_ERROR) {
        Terminate();
    }
    return(error);
}

SFCError Camera::Preview(Void)
{
    SFCError error = SFERR_NO_ERROR;

    if (_camera != NULL) {
        _status = STATUS_PREVIEW;
        _getable = false;
        _previewable = false;
        _pausable = false;
        _recordable = false;

        // vr[sBʂ̓R[obNB
        error = _camera->Preview();
    }
    else {
        error = EFAILED;
    }
    return(error);
}

SFCError Camera::Pause(Void)
{
    SFCError error = SFERR_NO_ERROR;

    if (_camera != NULL) {
        _status = STATUS_PAUSE;
        _previewable = false;
        _pausable = false;
        _recordable = false;
        // XgbvsBʂ̓R[obNB
        error = _camera->Stop();
    }
    else {
        error = EFAILED;
    }
    return(error);
}

SFCError Camera::Record(Void)
{
    SFCError error = SFERR_NO_ERROR;

    if (_camera != NULL) {
        _status = STATUS_RECORD;
        _previewable = false;
        _pausable = false;
        _recordable = false;
        // XgbvsBʂ̓R[obNB
        error = _camera->Stop();
    }
    else {
        error = EFAILED;
    }
    return(error);
}

Void Camera::Suspend(Void)
{
    if (_camera != NULL) {
        _status = STATUS_TERMINATE;
        _previewable = false;
        _pausable = false;
        _recordable = false;
        // XgbvsB{ʂ̓R[obN
        // ɔjȂ΍sȂ̂ŁA̎ŗǂB
        _camera->Stop();
        _camera.Release();
    }
    return;
}

SFCError Camera::Resume(Void)
{
    AEESize size;
    SFCError error = SFERR_NO_ERROR;

    if ((_camera = SFBCamera::NewInstance()) != NULL) {
        _status = STATUS_INITIALIZE;
        _previewable = true;
        _pausable = false;
        _recordable = false;
        // R[obNnho^B
        error = _camera->RegisterNotify(OnCameraSHP, this);
        if (error == SFERR_NO_ERROR) {
            size.cx = 240;
            size.cy = 320;
            error = _camera->SetSize(&size);
            if (error == SFERR_NO_ERROR) {
                error = _camera->SetDisplaySize(&size);
            }
        }
    }
    else {
        error = ENOMEMORY;
    }
    if (error != SFERR_NO_ERROR) {
        Suspend();
    }
    return(error);
}

Void Camera::Terminate(Void)
{
    // TXyhƓsB
    Suspend();
    if (_bitmap != NULL) {
        _bitmap.Release();
    }
    return;
}

SFBBitmapSmp& Camera::GetBitmap(Void)
{
    return(_bitmap);
}

Bool Camera::IsValid(Void) const
{
    return(_camera != NULL);
}

Bool Camera::IsGetable(Void) const
{
    return(_getable);
}

Bool Camera::IsPreviewable(Void) const
{
    return(_previewable);
}

Bool Camera::IsPausable(Void) const
{
    return(_pausable);
}

Bool Camera::IsRecordable(Void) const
{
    return(_recordable);
}

Void OnCameraSHP(VoidPtr reference, AEECameraNotify* notify)
{
    CameraPtr(reference)->OnCamera(notify);
    return;
}

Void Camera::OnCamera(AEECameraNotify* notify)
{
    SFXGraphicsPtr graphics;
    SFBBitmapSmp bitmap;

    switch (notify->nStatus) {
        case CAM_STATUS_START:
            if (_status == STATUS_PREVIEW) {
                _pausable = true;
                _recordable = true;
                _spp(_status, SFERR_NO_ERROR, _reference);
            }
            break;
        case CAM_STATUS_FRAME:
            switch (_status) {
                case STATUS_PREVIEW:
                    // vr[摜JfoCXAe|rbg}bv
                    // Rs[B
                    if (_camera->GetFrame(&bitmap) == SFERR_NO_ERROR) {
                        _bitmap->BltIn(SFXRectangle(0, 0, _rect.GetSize()), bitmap, _rect.GetOrigin());
                    }
                    _getable = true;
                    _spp(STATUS_FRAME, SFERR_NO_ERROR, _reference);

                    // 擾rbg}bvʂɕ`悷B
                    if ((graphics = SFXGraphics::GetInstance()) != NULL) {
                        graphics->BitBlt(_rect, _bitmap, SFXGrid(0, 0));
                        graphics->Update();
                    }
                    break;
                /*default:
                    break;*/
            }
            break;
        case CAM_STATUS_DONE:
            switch (_status) {
                case STATUS_PAUSE:
                    _previewable = true;
                    _spp(_status, SFERR_NO_ERROR, _reference);
                    break;
                case STATUS_RECORD:
                    _status = STATUS_ENCODE;
                    // xGR[hݒ肷B
                    if (_camera->DeferEncode(true) == SFERR_NO_ERROR) {
                        // XibvVbg[hsBʂ̓R[obNB
                        if (_camera->RecordSnapshot() != SFERR_NO_ERROR) {
                            _spp(_status, EFAILED, _reference);
                        }
                    }
                    else {
                        _spp(_status, EFAILED, _reference);
                    }
                    break;
                case STATUS_ENCODE:
                    // XibvVbg摜JfoCXA
                    // e|rbg}bvɃRs[B
                    if (_camera->GetFrame(&bitmap) == SFERR_NO_ERROR) {
                        _bitmap->BltIn(SFXRectangle(0, 0, _rect.GetSize()), bitmap, _rect.GetOrigin());
                    }
                    _getable = true;
                    _previewable = true;
                    _spp(_status, SFERR_NO_ERROR, _reference);
                    break;
                /*default:
                    break;*/
            }
            break;
        case CAM_STATUS_FAIL:
        case CAM_STATUS_ABORT:
            _spp(_status, EFAILED, _reference);
            break;
        /*default:
            break;*/
    }
    return;
}
